Package: V8
Type: Package
Title: Embedded JavaScript and WebAssembly Engine for R
Version: 4.1.0
Authors@R: person("Jeroen", "Ooms", role = c("aut", "cre"), email = "jeroen@berkeley.edu",
    comment = c(ORCID = "0000-0002-4035-0289"))
Description: An R interface to V8: Google's open source JavaScript and WebAssembly 
    engine. This package can be compiled either with V8 version 6 and up or NodeJS
    when built as a shared library.
License: MIT + file LICENSE
URL: https://github.com/jeroen/v8 (devel) https://v8.dev (upstream)
BugReports: https://github.com/jeroen/v8/issues
SystemRequirements: V8 engine version 6+ is needed for ES6 and WASM
        support. On Linux you can build against libv8-dev (Debian) or
        v8-devel (Fedora). We also provide static libv8 binaries for
        most platforms, see the README for details.
NeedsCompilation: yes
VignetteBuilder: knitr
Imports: Rcpp (>= 0.12.12), jsonlite (>= 1.0), curl (>= 1.0), utils
LinkingTo: Rcpp
Suggests: testthat, knitr, rmarkdown
RoxygenNote: 7.1.2
Language: en-US
Encoding: UTF-8
Packaged: 2022-02-06 22:38:13 UTC; jeroen
Author: Jeroen Ooms [aut, cre] (<https://orcid.org/0000-0002-4035-0289>)
Maintainer: Jeroen Ooms <jeroen@berkeley.edu>
Repository: CRAN
Date/Publication: 2022-02-06 23:10:02 UTC
Built: R 4.0.5; x86_64-w64-mingw32; 2022-02-07 03:37:35 UTC; windows
Archs: x64
