Package: V8
Type: Package
Title: Embedded JavaScript and WebAssembly Engine for R
Version: 4.4.2
Authors@R: c(
    person("Jeroen", "Ooms", role = c("aut", "cre"), email = "jeroen@berkeley.edu",
      comment = c(ORCID = "0000-0002-4035-0289")),
    person("Jan Marvin", "Garbuszus", role = "ctb"))
Description: An R interface to V8 <https://v8.dev>: Google's open source JavaScript
    and WebAssembly engine. This package can be compiled either with V8 version 6 
    and up or NodeJS when built as a shared library.
License: MIT + file LICENSE
URL: https://jeroen.r-universe.dev/V8
BugReports: https://github.com/jeroen/v8/issues
SystemRequirements: V8 engine version 6+ is needed for ES6 and WASM
        support. On Linux you can build against libv8-dev (Debian) or
        v8-devel (Fedora). We also provide static libv8 binaries for
        most platforms, see the README for details.
NeedsCompilation: yes
VignetteBuilder: knitr
Imports: Rcpp (>= 0.12.12), jsonlite (>= 1.0), curl (>= 1.0), utils
LinkingTo: Rcpp
Suggests: testthat, knitr, rmarkdown
RoxygenNote: 7.2.3
Language: en-US
Encoding: UTF-8
Biarch: true
Packaged: 2024-02-15 11:17:46 UTC; jeroen
Author: Jeroen Ooms [aut, cre] (<https://orcid.org/0000-0002-4035-0289>),
  Jan Marvin Garbuszus [ctb]
Maintainer: Jeroen Ooms <jeroen@berkeley.edu>
Repository: CRAN
Date/Publication: 2024-02-15 15:00:02 UTC
Built: R 4.4.0; aarch64-apple-darwin23.2.0; 2024-05-03 08:42:39 UTC; unix
