# Generated by roxygen2: do not edit by hand

S3method("$",V8)
S3method("[",V8)
S3method("[[",V8)
S3method(names,V8)
S3method(print,V8)
export(JS)
export(engine_info)
export(new_context)
export(v8)
export(wasm)
export(wasm_features)
import(Rcpp)
importFrom(curl,curl)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(utils,head)
importFrom(utils,loadhistory)
importFrom(utils,rc.options)
importFrom(utils,savehistory)
importFrom(utils,tail)
useDynLib(V8, .registration = TRUE)
