IF "%CONDA_BUILD%" == "" (
    call D:\bld\r-v8_1605358613221\work\build_env_setup.bat
)
REM ===== end generated header =====
mkdir %PREFIX%\lib\R\library
robocopy /E . "%PREFIX%\lib\R\library\V8"
if %ERRORLEVEL% NEQ 1 exit 1
exit 0